#include <stdio.h>
#include <iostream.h>

#include "generic.h"
#include "gui_muse.h"
#include "gui_pixmaps.h"
#include "jmixer.h"
#include "libsync/synchronous.h"
#include <FL/fl_ask.H>
#include <FL/fl_file_chooser.H>

static LTI::Synchronous *dealer;
static Stream_mixer *mixer;


/* where to store volumes for the talk func */
static float store_vol[MAX_CHANNELS];

/* resolves the bitrate widget choice */
inline int solve_bps(int bps_choice) {
  int bps = 0;

  switch(bps_choice) {
  case 0:
    bps = 8;
    break;
  case 1:
    bps = 16;
    break;
  case 2:
    bps = 24;
    break;
  case 3:
    bps = 32;
    break;
  case 4:
    bps = 56;
    break;
  case 5:
    bps = 64;
    break;
  case 6:
    bps = 96;
    break;
  case 7:
    bps = 128;
    break;
  }
  return(bps);
}

void init_callbacks(Stream_mixer *mix, const LTI::Synchronous *deal) {
  mixer = mix;
  dealer = (LTI::Synchronous *)deal;
}

void cb_set_channel(Fl_Widget *o, void *ch) {
  
  /* playlist starts from 0
     Fl browser value starts from 1 */
  int song = ((Fl_Browser *)o)->value();
  
  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();
  
  if(mixer->set_channel(*(int*)ch,(song-1))) {
    ((Channel*)(o->parent()->user_data()))->timeline_slider->value(0.0);
    ((Channel*)(o->parent()->user_data()))->time->value("00:00:00");
    ((Channel*)(o->parent()->user_data()))->name->value(((Channel*)(o->parent()->user_data()))->playlist_browser->text(song));
  }
}

void cb_play_channel(Fl_Widget *o, void *ch) {

  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();
  mixer->play_channel(*(int*)ch);
  
}

void cb_stop_channel(Fl_Widget *o, void *ch) {
  
  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();

  if(!mixer->stop_channel(*(int*)ch)) {
    char temp[100];
    sprintf(temp,"Something weird happened trying to stop channel [%u]",*(int*)ch);
    fl_alert(temp);
  } else {
    ((Channel*)(o->parent()->user_data()))->timeline_slider->value(0.0);
    ((Channel*)(o->parent()->user_data()))->time->value("00:00:00");
  }
}

void cb_pause_channel(Fl_Widget *o, void *ch) {

  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();

  mixer->pause_channel(*(int*)ch);
}

void cb_set_volume(Fl_Widget *o, void *ch) {

  float vol = ((Fl_Slider *)o)->value();

  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();

  mixer->set_volume(*(int*)ch,vol);
}

void cb_set_position(Fl_Widget *o, void *ch) {
  
  float pos = ((Fl_Slider *)o)->value();
  
  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();
  
  if(!mixer->set_position(pos,*(int*)ch)) {
    char temp[100];
    sprintf(temp,"Urg! it seems i can't seek position on channel [%u]",*(int*)ch);
    fl_alert(temp);
  }
}

void cb_set_playmode(Fl_Widget *o, void *ch) {
  int i = *(int*)ch;

  switch (mixer->xach[i].playmode)
    {
    case 0: /* LOOP -> PLAY */
      o->label("play");
      mixer->xach[i].playmode = mixer->xach[i].PLAY;
      break;
    case 1: /* CONT -> LOOP */
      o->label("loop");
      mixer->xach[i].playmode = mixer->xach[i].LOOP;
      break;
    case 2: /* PLAY -> CONT */
      o->label("cont");
      mixer->xach[i].playmode = mixer->xach[i].CONT;
      break;
    }

}

void cb_set_icecast(Fl_Widget *o) {
  Amuse *tgui;
  tgui = (Amuse*)(o->parent()->parent()->parent()->user_data());
  
  if(((Fl_Light_Button *)o)->value()==0) { /* stop the stream */
    
    dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
    dealer->wait();
    mixer->stop_icecast();
    
    tgui->ip_input->activate();
    tgui->port_input->activate();
    tgui->mount_input->activate();
    tgui->name_input->activate();
    tgui->url_input->activate();
    tgui->description_input->activate();
    tgui->pass_input->activate();
    tgui->connect_button->label("connect");
    tgui->bitrate_choice->activate();
    tgui->login_choice->activate();

  } else { /* set the stream */
    
    char *ip, *port, *mount, *pass, *name, *url, *desc;
    int bps, login;
    unsigned long int nport;

    ip = (char*)tgui->ip_input->value();
    port = (char*)tgui->port_input->value();
    nport = strtol(port,(char **)NULL,10);
    
    mount = (char*)tgui->mount_input->value();
    pass = (char*)tgui->pass_input->value();
    bps = tgui->bitrate_choice->value();

    name = (char*)tgui->name_input->value();
    url = (char*)tgui->url_input->value();
    desc = (char*)tgui->description_input->value();

    bps = solve_bps(bps);
    login = tgui->login_choice->value();
    
    dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
    dealer->wait();

    if(mixer->set_icecast(ip,nport,pass,mount,bps, name, url, desc, login))
      ((Fl_Light_Button *)o)->set();
    else
      ((Fl_Light_Button *)o)->clear();
  }

}

void cb_set_talk(Fl_Widget *o) {
  Amuse *tgui;
  int i;
  float temp_vol[MAX_CHANNELS];

  tgui = (Amuse *)(o->parent()->user_data());

  if(((Fl_Light_Button *)o)->value()==1) { 
    /* talk is not active; activate it */  
    for (i = 0; i < MAX_CHANNELS; i++) {
      store_vol[i] = tgui->chan[i]->volume_slider->value();
      if(store_vol[i]<TALK_VOLUME)
	temp_vol[i] = 0.0;
      else
	temp_vol[i] = TALK_VOLUME;
      tgui->chan[i]->volume_slider->value(temp_vol[i]);
    }

    dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
    dealer->wait();
    mixer->set_all_volumes(&temp_vol[0]);

  } else {   
    /* deactivate talk : bring volumes back up */
    for (i = 0; i < MAX_CHANNELS; i++)
      tgui->chan[i]->volume_slider->value(store_vol[i]);
    
    dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
    dealer->wait();
    mixer->set_all_volumes(&store_vol[0]);
    
  }

}

void cb_set_linein(Fl_Widget *o) {
  bool res;

  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();

  res = mixer->set_live(((Fl_Light_Button *)o)->value());

  if(res)
    ((Fl_Light_Button *)o)->set();
  else
    ((Fl_Light_Button *)o)->clear();
}

void cb_set_lineout(Fl_Widget *o) {
  bool res;

  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
  dealer->wait();

  res = mixer->set_lineout(((Fl_Light_Button *)o)->value());
  
  if(res)
    ((Fl_Light_Button *)o)->set();
  else
    ((Fl_Light_Button *)o)->clear();
  
}

void cb_save_file(Fl_Widget *o) {
  const char *filename;
  Amuse *tgui;

  tgui = (Amuse*)(o->parent()->parent()->parent()->user_data());
  filename = tgui->filesave_input->value();

  if (((Fl_Light_Button *)o)->value() == 1)
    {
      if ((filename[0] == '\0') || (filename[0] ==  ' '))   
	{
	  tgui->status_main->value("you need to type in a filename");
	  tgui->filesave_button->value(0);
	} else {
	  int res, bps;

	  /* retrieve the selected bps */
	  bps = solve_bps(tgui->bitrate_choice->value());

	  /* input syntax ok, let's start recording */
	  dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
	  dealer->wait();
	  res = mixer->set_mp3file((char*)filename, bps);
	  /* if res -1 should notice error */

	  tgui->filebrowse_button->deactivate();
	  tgui->filesave_input->deactivate();
	  tgui->status_main->value(filename);
	}
    }
  if (((Fl_Light_Button *)o)->value() == 0)
    {
      /* let's shutdown file recording */
      dealer->select.call(0,mixer,Stream_mixer::id_stream_mixer);
      dealer->wait();
      mixer->stop_mp3file();

      tgui->filebrowse_button->activate();
      tgui->filesave_input->activate();
      tgui->status_main->value(" ");
      
    }
}

void cb_browse_file(Fl_Widget *o) {
    char *filename;
    Amuse *tgui;
    tgui = (Amuse*)(o->parent()->parent()->parent()->user_data());
    filename = fl_file_chooser("Pick a file", "*", "");
    tgui->filesave_input->value(filename); 
}

Fl_Window *infowindow = NULL;
inline void infoz() {

  infowindow = new Fl_Window(280, 220, "Info about MuSE");
  Fl_Box *box = new Fl_Box(35, 15, 250, 30, "MuSE - Multiple Streaming Engine\nhttp://muse.dyne.org");
  box->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  box->labelsize(12);
  box->labelfont(1);
  Fl_Box *names = new Fl_Box(35, 70, 250, 50, "2000 by Authors:\n\n  denis roio a.k.a. jaromil - MuSE\n           &\n  august black - MuSE gui");
  names->align(FL_ALIGN_LEFT|FL_ALIGN_TOP|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  names->labelsize(12);
  names->labelfont(1);
  
  Fl_Box *credits = new Fl_Box(15,170,260,30, "development of this software is supported by:\nSERVUS  http://www.servus.at  (feb-nov 2000)");
  credits->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  credits->labelsize(10);
  credits->labelfont(1);

  Fl_Button *but = new Fl_Button(10, 15, 20, 20);
  but->box(FL_FLAT_BOX);
  but->down_box(FL_FLAT_BOX);
  pixmap_peach.label(but);

 
  infowindow->end();
  infowindow->show();
}
void cb_open_info(Fl_Widget *o) {
  if(infowindow!=NULL) {
    if(!infowindow->shown())
      infoz();
  } else
    infoz();
}

void gui_quit(Fl_Menu_* o, void* v) {
  ((Amuse*)(o->parent()->user_data()))->quit = true;
}
